@@ -4,6 +4,7 @@ import boto3
 import jinja2
 import logging
 import os
+import random
 import re
 import redis
 import requests
@@ -251,7 +252,8 @@ class Retemplate(object):
             'group': None,
             'chmod': 600,
             'onchange': None,
-            'frequency': 60
+            'frequency': 60,
+            'random_offset_max': None
         }
         self.settings.update(kwargs)
         self.vars = dict()
@@ -368,6 +370,13 @@ class Retemplate(object):
             password: rtpl://secrets/{<env>}.the_password
         '''
 
+        # Optionally stagger renders to avoid "thundering herd" effect when running this from many hosts
+        if self.settings.get('random_offset_max'):
+            offset = random.randint(1, self.settings['random_offset_max'])
+            logging.info('Waiting an additional {} seconds to render target {}'.format(
+                offset, self.target))
+            sleep(offset)
+
         logging.info('Rendering template {} for target {}'.format(self.template, self.target))
         try:
             tpl = self.read_template()
@@ -401,10 +410,12 @@ class Retemplate(object):
         '''
 
         try:
+            logging.info("Writing {}; setting ownership to {}:{} and mode to {}".format(
+                self.target, self.settings['owner'], self.settings['group'], self.settings['chmod']))
             with open(self.target, 'w') as fh:
                 fh.write(content)
             shutil.chown(self.target, user=self.settings['owner'], group=self.settings['group'])
-            os.chmod(self.target, self.settings['chmod'])
+            subprocess.run([ 'chmod', settings['chmod'], self.target ])
             return True
         except IOError:
             logging.error('Cannot write target file {}'.format(self.target))
